package com.weebkun.speedq_printer;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.IBinder;
import android.util.Log;

import net.posprinter.posprinterface.IMyBinder;
import net.posprinter.posprinterface.UiExecute;
import net.posprinter.service.PosprinterService;
import net.posprinter.utils.DataForSendToPrinterPos58;
import net.posprinter.utils.DataForSendToPrinterPos80;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;


/**
 * class to manage bluetooth discovery, connections, and devices
 */
public class BluetoothManager {
    private static final BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
    private static IMyBinder binder;

    private static final ServiceConnection conn = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            System.out.println("bound");
            //Bind successfully
            binder = (IMyBinder) iBinder;
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
        }
    };

    public static Device[] getPairedDevices() {
        Device[] devices = adapter.getBondedDevices().stream().map(Device::new).toArray(Device[]::new);
        for(Device device : devices) {
            System.out.println(device.name);
        }
        return devices;
    }

    public static void connect(String mac) {
        System.out.println("connect");
        binder.connectBtPort(mac, new UiExecute() {
            @Override
            public void onsucess() {
                Log.i("bt", "connected to printer");
                binder.write(DataForSendToPrinterPos80.openOrCloseAutoReturnPrintState(0x1f), new UiExecute() {
                    @Override
                    public void onsucess() {
                        binder.acceptdatafromprinter(new UiExecute() {
                            @Override
                            public void onsucess() {
                            }

                            @Override
                            public void onfailed() {
                            }
                        });
                    }

                    @Override
                    public void onfailed() {
                    }
                });
            }

            @Override
            public void onfailed() {
                Log.e("bt", "cannot connect");
            }
        });
    }

    public static void bindService(Context context) {
        System.out.println("bind");
        context.bindService(new Intent(context, PosprinterService.class), conn, Context.BIND_AUTO_CREATE);
    }

    public static void printText(String text){

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    List<byte[]> list = new ArrayList<>();
                    //initialize the printer
//                            list.add( DataForSendToPrinterPos58.initializePrinter());
                    list.add(DataForSendToPrinterPos58.initializePrinter());
                    byte[] data1 = stringToBytes(text);
                    list.add(data1);
                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static byte[] stringToBytes(String str){
        byte[] data = null;
        try {
            data = str.getBytes("gbk");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        return data;
    }

    public static void printCenter(String message, int size){
        System.out.println("printCenter");
        // Print config "mode"
        byte[] cc = new byte[] { 0x1B, 0x21, 0x03 }; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[] { 0x1B, 0x21, 0x08 }; // 1- only bold text
        byte[] bb2 = new byte[] { 0x1B, 0x21, 0x20 }; // 2- bold with medium text
        byte[] bb3 = new byte[] { 0x1B, 0x21, 0x10 }; // 3- bold with large text
        byte[] bb4 = new byte[] { 0x1B, 0x21, 0x30 }; // 4- strong text

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        switch (size) {
            case 0:
                list.add(cc);
                break;
            case 1:
                list.add(bb);
                break;
            case 2:
                list.add(bb2);
                break;
            case 3:
                list.add(bb3);
                break;
            case 4:
                list.add(bb4);
                break;
        }

        list.add(PrinterCommands.ESC_ALIGN_CENTER);
        list.add(message.getBytes());
        list.add(PrinterCommands.FEED_LINE);

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void printLeft(String message, int size){
        System.out.println("printLeft");
        // Print config "mode"
        byte[] cc = new byte[] { 0x1B, 0x21, 0x03 }; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[] { 0x1B, 0x21, 0x08 }; // 1- only bold text
        byte[] bb2 = new byte[] { 0x1B, 0x21, 0x20 }; // 2- bold with medium text
        byte[] bb3 = new byte[] { 0x1B, 0x21, 0x10 }; // 3- bold with large text
        byte[] bb4 = new byte[] { 0x1B, 0x21, 0x30 }; // 4- strong text

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        switch (size) {
            case 0:
                list.add(cc);
                break;
            case 1:
                list.add(bb);
                break;
            case 2:
                list.add(bb2);
                break;
            case 3:
                list.add(bb3);
                break;
            case 4:
                list.add(bb4);
                break;
        }

        list.add(PrinterCommands.ESC_ALIGN_LEFT);
        list.add(message.getBytes());
        list.add(PrinterCommands.FEED_LINE);

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void printRight(String message, int size){
        System.out.println("printRight");
        // Print config "mode"
        byte[] cc = new byte[] { 0x1B, 0x21, 0x03 }; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[] { 0x1B, 0x21, 0x08 }; // 1- only bold text
        byte[] bb2 = new byte[] { 0x1B, 0x21, 0x20 }; // 2- bold with medium text
        byte[] bb3 = new byte[] { 0x1B, 0x21, 0x10 }; // 3- bold with large text
        byte[] bb4 = new byte[] { 0x1B, 0x21, 0x30 }; // 4- strong text

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        switch (size) {
            case 0:
                list.add(cc);
                break;
            case 1:
                list.add(bb);
                break;
            case 2:
                list.add(bb2);
                break;
            case 3:
                list.add(bb3);
                break;
            case 4:
                list.add(bb4);
                break;
        }

        list.add(PrinterCommands.ESC_ALIGN_RIGHT);
        list.add(message.getBytes());
        list.add(PrinterCommands.FEED_LINE);

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

    public static void print3Column(String msg1, String msg2, String msg3, int size) {
        System.out.println("print3Column");
        byte[] cc = new byte[] { 0x1B, 0x21, 0x03 }; // 0- normal size text
        // byte[] cc1 = new byte[]{0x1B,0x21,0x00}; // 0- normal size text
        byte[] bb = new byte[] { 0x1B, 0x21, 0x08 }; // 1- only bold text
        byte[] bb2 = new byte[] { 0x1B, 0x21, 0x20 }; // 2- bold with medium text
        byte[] bb3 = new byte[] { 0x1B, 0x21, 0x10 }; // 3- bold with large text
        byte[] bb4 = new byte[] { 0x1B, 0x21, 0x30 }; // 4- strong text

        List<byte[]> list = new ArrayList<>();
        list.add(DataForSendToPrinterPos58.initializePrinter());

        switch (size) {
            case 0:
                list.add(cc);
                break;
            case 1:
                list.add(bb);
                break;
            case 2:
                list.add(bb2);
                break;
            case 3:
                list.add(bb3);
                break;
            case 4:
                list.add(bb4);
                break;
        }
        list.add(PrinterCommands.ESC_ALIGN_CENTER);
        String line = String.format("%-10s %10s %10s %n", msg1, msg2, msg3);
        list.add(line.getBytes());

        binder.writeDataByYouself(
                new UiExecute() {
                    @Override
                    public void onsucess() {

                    }

                    @Override
                    public void onfailed() {

                    }
                }, () -> {

                    //should add the command of print and feed line,because print only when one line is complete, not one line, no print
                    list.add(DataForSendToPrinterPos58.printAndFeedLine());
                    return list;
                });
    }

}
